#include "PWMRegs.h"
#include "types.h"
#include "Utils.h"

/*
 * PWM Timer is residing on the APB bus. 
 * For the time being - we assume that the clock used to drive this bus has 
 * 50MHZ frequency
*/
  
static U32 PCLK = 50000000;  


void Timer_Bdelay_Milli (unsigned long Millis)
{
	U32 Prev;
	U32 Current;
	U32 ElapsedSteps = 0;
	U32 StepsInOneMilli = (PCLK>>3)/1000;
	U32 TotalSteps = StepsInOneMilli * Millis;

	

	/*
	 * Set timer step period T for Timer 4 - Divide PCLK by 8
	*/	
	*TCFG0 &= ~(0xff<<8);
	*TCFG0 |= 3<<8;			        //prescaler = 3+1
	*TCFG1 &= ~(0xf<<16);
	*TCFG1 |= TIMER4_MUX_1_2;		//mux = 1/2

	/*
	 * Tooth period cT is set by multiplying T by the number of countown steps.
	*/	
	*TCNTB4 = StepsInOneMilli;
	

	/*
	 * Activate timer 4.
	*/	
	
	*TCON |= (TIMER4_START | TIMER4_MAN_UPDATE | TIMER4_AUTO_RELOAD);	//autoload, update TCNTB4, start timer 4
	*TCON &= ~(TIMER4_MAN_UPDATE);		//clear manual update bit
	
	Prev = *TCNTO4;

	while (ElapsedSteps < TotalSteps)
	{
		Current = *TCNTO4;
		if (Current <= Prev)
		{
			ElapsedSteps += (Prev - Current);
		}
		else
		{
			ElapsedSteps += (StepsInOneMilli - (Current - Prev));
		}
		
		Prev = Current;
	}

	/* Stop the timer */	
	*TCON &= ~(TIMER4_START | TIMER4_AUTO_RELOAD);	
}


void TestBdelayMilli ()
{
	U32 Val;
	
	Util_Printf ("Enter the delay in milliseconds (decimal)...");	
	Val = Util_GetIntNum ();
	Timer_Bdelay_Milli (Val);	
}

